==> evrete-core-3.2.01-sources.jar: org/evrete/api/StatefulSession.java <==
package org.evrete.api;

import java.util.concurrent.Future;
import java.util.concurrent.RejectedExecutionException;
import java.util.function.BiConsumer;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Predicate;

/**
 * The StatefulSession interface represents a stateful rule session.
 */
public interface StatefulSession extends RuleSession<StatefulSession>, AutoCloseable {

    /**
     * Fire all rules.
     */
    StatefulSession fire();

    /**
     * <p>
     * Updates a fact that already exists in the working memory
     * </p>
     *
     * @param handle   fact handle, previously assigned to original fact
     * @param newValue an updated version of the fact
     * @return this instance
     */
    StatefulSession update(FactHandle handle, Object newValue);

    /**
     * <p>
     * Deletes a fact from working memory.
     * </p>
     *
     * @param handle fact handle
     * @return this instance
     */
    StatefulSession delete(FactHandle handle);

    /**
     * <p>
     * Returns fact by its handle.
     * </p>
     *
     * @param handle fact handle
     * @param <T>    type of the fact (use {@code Object}  or wildcard if type is unknown)
     * @return fact or {@code null} if fact is not found
     */
    <T> T getFact(FactHandle handle);

    /**
     * <p>
     * Fires session asynchronously and returns a Future representing the session execution
     * status.
     * </p>
     *
     * @param result the result to return by the Future
     * @param <T>    result type parameter
     * @return a Future representing pending completion of the {@link #fire()} command
     * @throws RejectedExecutionException if the task cannot be
     *                                    scheduled for execution
     */
    <T> Future<T> fireAsync(T result);

    /**
     * <p>
     * Same as {@link #fireAsync(Object)}, with the Future's get method will returning
     * the session itself.
     * </p>
     *
     * @return a Future representing pending completion of the {@link #fire()} command.
     * @throws RejectedExecutionException if the task cannot be
     *                                    scheduled for execution
     * @see #fireAsync(Object)
     */
    default Future<StatefulSession> fireAsync() {
        return fireAsync(this);
    }

    /**
     * <p>
     * Closes the session and destroys its memory. A closed session can not be reused.
     * </p>
     */
    void close();

    /**
     * <p>
     * This method clears session's working memory and its beta-memory nodes as if the session
     * had never been used.
     * </p>
     */
    void clear();


    /**
     * <p>
     * A full-scan memory inspection method.
     * </p>
     *
     * @param consumer consumer for the facts
     * @return this instance
     */
    StatefulSession forEachFact(BiConsumer<FactHandle, Object> consumer);

    default StatefulSession forEachFact(BiPredicate<FactHandle, Object> filter, BiConsumer<FactHandle, Object> consumer) {
        return forEachFact((factHandle, o) -> {
            if (filter.test(factHandle, o)) {
                consumer.accept(factHandle, o);
            }
        });
    }

    default StatefulSession forEachFact(Consumer<Object> consumer) {
        return forEachFact((factHandle, o) -> consumer.accept(o));
    }

    /**
     * <p>
     * A memory inspection method that accepts fact type as an argument.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @return this instance
     * @throws ClassCastException       if a working memory object can not be cast to the specified type
     * @throws IllegalArgumentException if no such type exists
     * @see TypeResolver
     */
    default <T> StatefulSession forEachFact(Class<T> type, Consumer<T> consumer) {
        return forEachFact(type.getName(), consumer);
    }

    /**
     * <p>
     * A memory inspection method that accepts fact type as an argument. Type name can be either
     * a class name or a name of explicitly declared type. In the latter case, the generic type
     * parameter {@code T} must match the declared type's Java type (see {@link Type#getJavaType()})
     * </p>
     *
     * @param type     type name
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @return this instance
     * @throws ClassCastException       if a working memory object can not be cast to the specified type
     * @throws IllegalArgumentException if no such type exists
     * @see TypeResolver
     */
    <T> StatefulSession forEachFact(String type, Consumer<T> consumer);

    /**
     * <p>
     * A filtering version of the {@link #forEachFact(Class, Consumer)} method.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @param filter   filtering predicate
     * @return this instance
     * @see #forEachFact(Class, Consumer)
     */
    default <T> StatefulSession forEachFact(Class<T> type, Predicate<T> filter, Consumer<T> consumer) {
        return forEachFact(type, t -> {
            if (filter.test(t)) {
                consumer.accept(t);
            }
        });
    }

    /**
     * <p>
     * A filtering version of the {@link #forEachFact(String, Consumer)} method.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @param filter   filtering predicate
     * @return new stateful session instance
     * @see #forEachFact(String, Consumer)
     */
    @SuppressWarnings("unchecked")
    default <T> StatefulSession forEachFact(String type, Predicate<T> filter, Consumer<T> consumer) {
        return forEachFact(type, o -> {
            T t = (T) o;
            if (filter.test(t)) {
                consumer.accept(t);
            }
        });
    }

    default StatefulSession insertAndFire(Iterable<?> objects) {
        insert0(objects, true);
        return fire();
    }

    default StatefulSession insertAndFire(Object... objects) {
        insert0(objects, true);
        return fire();
    }

}

==> evrete-core-4.0.1-sources.jar: org/evrete/api/StatefulSession.java <==
package org.evrete.api;

import java.util.concurrent.Future;
import java.util.function.BiConsumer;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Predicate;

/**
 * The StatefulSession interface represents a stateful rule session.
 */
public interface StatefulSession extends RuleSession<StatefulSession>,  AutoCloseable {

    /**
     * Fire all rules.
     */
    StatefulSession fire();


    /**
     * Fires the session asynchronously and returns a Future representing the session's execution status.
     *
     * @param result the result to return by the Future
     * @param <T> the result type parameter
     * @return a Future representing the pending completion of the {@link #fire()} command
     * @deprecated Use external tools like {@link java.util.concurrent.ExecutorService#submit(Runnable, Object)}.
     */
    @Deprecated
    default <T> Future<T> fireAsync(T result) {
        throw new UnsupportedOperationException("Deprecated");
    }

    /**
     * Same as {@link #fireAsync(Object)}, but the Future's get method will return
     * the session itself.
     *
     * @return a Future representing pending completion of the {@link #fire()} command.
     * @deprecated Use external tools like {@link java.util.concurrent.ExecutorService#submit(Runnable, Object)}.
     */
    @Deprecated
    default Future<StatefulSession> fireAsync() {
        throw new UnsupportedOperationException("Deprecated");
    }

    /**
     * <p>
     * Closes the session and destroys its memory. A closed session can not be reused.
     * </p>
     */
    void close();

    /**
     * <p>
     * This method clears session's working memory and its beta-memory nodes as if the session
     * had never been used.
     * </p>
     */
    void clear();


    /**
     * <p>
     * A full-scan memory inspection method.
     * </p>
     *
     * @param consumer consumer for the facts
     * @return this instance
     */
    default StatefulSession forEachFact(BiConsumer<FactHandle, Object> consumer){
        streamFactEntries().forEach(entry -> consumer.accept(entry.getKey(), entry.getValue()));
        return this;
    }

    default StatefulSession forEachFact(BiPredicate<FactHandle, Object> filter, BiConsumer<FactHandle, Object> consumer) {
        return forEachFact((factHandle, o) -> {
            if (filter.test(factHandle, o)) {
                consumer.accept(factHandle, o);
            }
        });
    }

    default StatefulSession forEachFact(Consumer<Object> consumer) {
        return forEachFact((factHandle, o) -> consumer.accept(o));
    }

    /**
     * <p>
     * A memory inspection method that accepts fact type as an argument.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @return this instance
     * @throws ClassCastException       if a working memory object can not be cast to the specified type
     * @throws IllegalArgumentException if no such type exists
     * @see TypeResolver
     */
    default <T> StatefulSession forEachFact(Class<T> type, Consumer<T> consumer) {
        streamFacts(type).forEach(consumer);
        return this;
    }

    /**
     * <p>
     * A memory inspection method that accepts fact type as an argument. Type name can be either
     * a class name or a name of explicitly declared type. In the latter case, the generic type
     * parameter {@code T} must match the declared type's Java type (see {@link Type#getJavaClass()} )
     * </p>
     *
     * @param type     type name
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @return this instance
     * @throws ClassCastException       if a working memory object can not be cast to the specified type
     * @throws IllegalArgumentException if no such type exists
     * @see TypeResolver
     */
    default <T> StatefulSession forEachFact(String type, Consumer<T> consumer) {
        this.<T>streamFacts(type).forEach(consumer);
        return this;
    }

    /**
     * <p>
     * A filtering version of the {@link #forEachFact(Class, Consumer)} method.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @param filter   filtering predicate
     * @return this instance
     * @see #forEachFact(Class, Consumer)
     */
    default <T> StatefulSession forEachFact(Class<T> type, Predicate<T> filter, Consumer<T> consumer) {
        return forEachFact(type, t -> {
            if (filter.test(t)) {
                consumer.accept(t);
            }
        });
    }

    /**
     * <p>
     * A filtering version of the {@link #forEachFact(String, Consumer)} method.
     * </p>
     *
     * @param type     expected fact type
     * @param consumer consumer for the facts
     * @param <T>      expected java type for the provided type name
     * @param filter   filtering predicate
     * @return new stateful session instance
     * @see #forEachFact(String, Consumer)
     */
    @SuppressWarnings("unchecked")
    default <T> StatefulSession forEachFact(String type, Predicate<T> filter, Consumer<T> consumer) {
        return forEachFact(type, o -> {
            T t = (T) o;
            if (filter.test(t)) {
                consumer.accept(t);
            }
        });
    }

    default StatefulSession insertAndFire(Iterable<?> objects) {
        insert0(objects, true);
        return fire();
    }

    default StatefulSession insertAndFire(Object... objects) {
        insert0(objects, true);
        return fire();
    }

}
